<?php
/***********************************************************************************
 * X2Engine Open Source Edition is a customer relationship management program developed by
 * X2 Engine, Inc. Copyright (C) 2011-2019 X2 Engine Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by the
 * Free Software Foundation with the addition of the following permission added
 * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
 * IN WHICH THE COPYRIGHT IS OWNED BY X2ENGINE, X2ENGINE DISCLAIMS THE WARRANTY
 * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License along with
 * this program; if not, see http://www.gnu.org/licenses or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * 
 * You can contact X2Engine, Inc. P.O. Box 610121, Redwood City,
 * California 94061, USA. or at email address contact@x2engine.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "Powered by
 * X2 Engine" logo. If the display of the logo is not reasonably feasible for
 * technical reasons, the Appropriate Legal Notices must display the words
 * "Powered by X2 Engine".
 **********************************************************************************/




Yii::import ('application.components.sortableWidget.SortableWidget');
Yii::import('application.components.sortableWidget.SortableWidgetResizeBehavior');

/**
 * @package application.components
 */
class IframeWidget extends SortableWidget {

    public $canBeDeleted = true;

    public $defaultTitle = 'Website Viewer';

    public $sortableWidgetJSClass = 'IframeWidget';

    public $relabelingEnabled = true;

    public $viewFile = '_iframeWidget';

    public $template = '<div class="submenu-title-bar widget-title-bar">{widgetLabel}{closeButton}{minimizeButton}{settingsMenu}</div>{widgetContents}';

    private static $_JSONPropertiesStructure;

    public function getViewFileParams () {
        if (!isset ($this->_viewFileParams)) {
            $this->_viewFileParams = array_merge (
                parent::getViewFileParams (),
                array (
                    'url' => self::getJSONProperty (
                        $this->profile, 'url', $this->widgetType, $this->widgetUID),
                    'height' => self::getJSONProperty (
                        $this->profile, 'height', $this->widgetType, $this->widgetUID),
                )
            );
        }
        return $this->_viewFileParams;
    } 

    protected function getSettingsMenuContent () {
        $htmlStr = 
            '<div class="widget-settings-menu-content" style="display:none;">
                <ul>'. 
                    ($this->relabelingEnabled ? 
                    '<li class="relabel-widget-button">'.
                        Yii::t('app', 'Rename Widget').
                    '</li>' : '').
                    ($this->canBeDeleted ? 
                        '<li class="delete-widget-button">'.
                            Yii::t('app', 'Delete Widget').
                        '</li>' : '').
                    '<li class="change-url-button">'.
                        Yii::t('profile', 'Change URL').
                    '</li>
                </ul>
            </div>';
        if ($this->relabelingEnabled) {
            $htmlStr .= 
                '<div id="relabel-widget-dialog-'.$this->widgetUID.'" style="display: none;">
                    <div>'.Yii::t('app', 'Enter a new name:').'</div>  
                    <input class="new-widget-name">
                </div>';
        }
        if ($this->canBeDeleted) {
            $htmlStr .= 
                '<div id="delete-widget-dialog-'.$this->widgetUID.'" style="display: none;">
                    <div>'.
                        Yii::t('app', 'Performing this action will cause this widget\'s settings '.
                            'to be lost. This action cannot be undone.').
                    '</div>  
                </div>';
        }

        $htmlStr .= 
            '<div id="change-url-dialog-'.$this->widgetUID.'" class="change-url-dialog" 
              style="display: none;">'.
                '<div>'.Yii::t('profile', 'Enter a URL:').'</div>'.
                '<input class="iframe-url">'.
            '</div>';
        return $htmlStr;
    }


    /**
     * overrides parent method
     */
    public static function getJSONPropertiesStructure () {
        if (!isset (self::$_JSONPropertiesStructure)) {
            self::$_JSONPropertiesStructure = array_merge (
                parent::getJSONPropertiesStructure (),
                array (
                    'url' => '', 
                    'label' => Yii::t('app', 'Website Viewer'),
                    'height' => '200',
                    'hidden' => true
                )
            );
        }
        return self::$_JSONPropertiesStructure;
    }

    protected function getJSSortableWidgetParams () {
        if (!isset ($this->_JSSortableWidgetParams)) {
            $this->_JSSortableWidgetParams = array_merge (parent::getJSSortableWidgetParams (),
                array (
                    'enableResizing' => true,
                )
            );
        }
        return $this->_JSSortableWidgetParams;
    }

    /**
     * @return array translations to pass to JS objects 
     */
    protected function getTranslations () {
        if (!isset ($this->_translations )) {
            $this->_translations = array_merge (parent::getTranslations (), array (
                'dialogTitle'=> Yii::t('profile', 'Change URL'),
                'closeButton'=> Yii::t('profile', 'Close'),
                'selectButton'=> Yii::t('profile', 'Change'),
                'urlError'=> Yii::t('profile', 'URL cannot be blank'),
            ));
        }
        return $this->_translations;
    }

    /**
     * overrides parent method. Adds JS file necessary to run the setup script.
     */
    public function getPackages () {
        if (!isset ($this->_packages)) {
            $this->_packages = array_merge (
                parent::getPackages (),
                array (
                    'IframeWidgetJS' => array(
                        'baseUrl' => Yii::app()->request->baseUrl,
                        'js' => array(
                            'js/sortableWidgets/IframeWidget.js',
                        ),
                        'depends' => array ('SortableWidgetJS')
                    ),
                )
            );
        }
        return $this->_packages;
    }

    /**
     * Magic getter. Returns this widget's css
     * @return array key is the proposed name of the css string which should be passed as the first
     *  argument to yii's registerCss. The value is the css string.
     */
    protected function getCss () {
        if (!isset ($this->_css)) {
            $this->_css = array_merge (
                parent::getCss (),
                array (
                    'iframeWidgetCss' => "
                        #".get_called_class()."-widget-content-container {
                            padding-bottom: 1px;
                        }

                        .change-url-dialog p {
                            display: inline;
                            margin-right: 5px;
                        }

                        .default-text-container {
                            text-align: center;
                            position: absolute;
                            top: 0;
                            bottom: 0;
                            left: 0;
                            right: 0;
                        }

                        .default-text-container a {
                            height: 17%;
                            text-decoration: none;
                            font-size: 16px;
                            margin: auto;
                            position: absolute;
                            left: 0;
                            top: 0;
                            right: 0;
                            bottom: 0;
                            color: #222222 !important;
                        }
                    "
                )
            );
        }
        return $this->_css;
    }

}
?>
